#!/bin/bash

#   FILE: backup.sh -- Backup system to a CD-RW.
# AUTHOR: W. Michael Petullo <burn_baby_burn@flyn.org>
#   DATE: 20 SEP 1998
#
# Copyright (c) 1999 W. Michael Petullo <burn_baby_burn@flyn.org>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

. /etc/burn_baby_burn/bbb_functions

TYPE=$1
ERR_DEST=$2
SET=$3
VOL_ID="backup -- `date +'%d %b %Y'`"
USAGE="
Usage: ${0##*/} existing_media|new_media|cdr|<path> log|stderr <set>\n
\n
Allows one to perform a backup to CD-R[W] media.  The directories to burn
are specified in /etc/burn_baby_burn/burn_baby_burn.  This program can
make bootable (El Torito) backup media.  If you want to boot a kernel
off of a CD, make sure your CD-ROM driver and the ISO-9660 filesystem
are not kernel modules.  Specify BOOTABLE=true in you set configuration
file.
\n
An inittab and rc file are provided which try to provide a sane environment,
given the root disk is read-only.  Backup.sh insures these are placed
in appropriate places in the generated ISO-9660 filesystem. \n
\n
existing_media checks for the existance of the file backup in the root
directory of the media being burned to.  This can allow one to use
${0##*/} in a cron script, without having to worry about accidentally
destroying the wrong disk.\n
new_media makes no such check.\n
cdr does not make a check or attempt to blank the disc. \n
Anything else causes an image file to be written to the path given. \n
\n
log causes error messages to be logged using the logger command. \n
stderr causes error messages to be printed to stderr. \n
\n
set is the name of the set to burn.  An appropriate configuration \n
file must exist. \n"

trap 'rm -rf $TMP_DIR; rm -f /tmp/be_nice; exit 6' SIGHUP SIGINT SIGTERM
trap 'rm -rf $TMP_DIR; rm -f /tmp/be_nice' EXIT

if [ $# != 3 ]; then
	usage "$USAGE"
elif [ $ERR_DEST != log -a $ERR_DEST != stderr ]; then
	usage "$USAGE"
elif ! configured; then
	print_message $ERR_DEST "No config found in /etc or ~"
	exit 1
elif ! set_configured $SET; then
	print_message $ERR_DEST "No configuration for set $SET found"
	exit 1
fi

if [ $TYPE = cdr ]; then
	MEDIA_TYPE=cdr
elif [ $TYPE = existing_media -o $TYPE = new_media ]; then
	MEDIA_TYPE=cdrw
else
	MEDIA_TYPE=$TYPE
fi

# Okay, there are two problems with this /tmp/be_nice system.  
#
# One, this lockfile does not affect CPU intensive cron jobs that begin
# running before backup.sh does.  Because of this, it is important to
# structure your crontab file accordingly.  For example, the scripts
# in /etc/cron.daily seem to run every day at 0400.  It makes sense to
# run backup.sh at 0355, and wrap all of the scripts in /etc/cron.daily
# in do_nicely.sh.

touch /tmp/be_nice
touch /tmp/backup_failed
print_message $ERR_DEST "Backup start time: `date`"

if [ $TYPE = existing_media ]; then
	if ! disc_mounts; then
		print_message $ERR_DEST "WARNING: bad filesystem -- before burn"
		exit 2
	elif ! is_backup_disc $SET; then
		print_message $ERR_DEST "WARNING: incorrect disc"
		exit 4
	fi
else
	umount $CD_DEV 2> /dev/null	
fi

# Alrighty, everything should okay, lets give it a (LASER) shot -- heh heh.

# Although this copy is slow and seems redundant, it may be necessary
# to ensure data does not change from time of burn to time of verify.
if ! mktmpdir /tmp/backup_tmp_dir.XXXXXX; then
	print_message $ERR_DEST "WARNING: could not create temp. dir."
	exit 5
fi

copy_files $TMP_DIR "$SKIP_DIRS" "$COMPRESS_DIRS" "$BACKUP_DIRS"
compress_files $TMP_DIR "$SKIP_DIRS" "$COMPRESS_DIRS"

touch $TMP_DIR/backup.$SET

if ! burn_dir.sh "$VOL_ID" $TMP_DIR $MEDIA_TYPE $ERR_DEST; then
	print_message $ERR_DEST "WARNING: error burning to media"
	exit 8
fi

# Backup seems to be okay.
rm -f /tmp/backup_failed
print_message $ERR_DEST "Backup end time: `date`"
exit 0
